@@ -6,6 +6,7 @@
 do '../web-lib.pl';
 &init_config();
 do 'records-lib.pl';
+require '../ui-lib.pl';
 @extra_forward = split(/\s+/, $config{'extra_forward'});
 @extra_reverse = split(/\s+/, $config{'extra_reverse'});
 %is_extra = map { $_, 1 } (@extra_forward, @extra_reverse);
@@ -797,7 +798,11 @@ print "</td> </tr>\n";
 @v = @{$rec{'values'}};
 if ($type eq "A" || $type eq "AAAA") {
 	print "<tr> <td><b>$text{'value_A1'}</b></td>\n";
-	print "<td><input name=value0 size=20 value=\"$v[0]\"></td> </tr>\n";
+	print "<td><input name=value0 size=20 value=\"$v[0]\">\n";
+	if (@_ < 6 && $type eq "A") {
+		print &free_address_button("value0");
+		}
+	print "</td> </tr>\n";
 	if (@_ >= 6) {
 		print "<input type=hidden name=oldname ",
 		      "value=\"$rec{'name'}\">\n";
@@ -1173,27 +1178,38 @@ local $out = `$config{'named_path'} -help 2>&1`;
 return $out !~ /\[-f\]/ && $out !~ /\[-f\|/ ? $out : undef;
 }
 
+sub get_chroot
+{
+if ($config{'auto_chroot'}) {
+	local $out = `$config{'auto_chroot'} 2>/dev/null`;
+	$out =~ s/\r|\n//g;
+	return $out;
+	}
+else {
+	return $config{'chroot'};
+	}
+}
+
 # make_chroot(file)
 sub make_chroot
 {
-return $_[0] if (!$config{'chroot'});
+local $chroot = &get_chroot();
+return $_[0] if (!$chroot);
 return $_[0] if ($_[0] eq $config{'named_conf'} && $config{'no_chroot'});
-return $config{'chroot'}.$_[0];
+return $chroot.$_[0];
 }
 
-# has_ndc()
+# has_ndc(exclude-mode)
 # Returns 2 if rndc is installed, 1 if ndc is instaled, or 0
 sub has_ndc
 {
-if ($config{'rndc_cmd'} =~ /^(\S+)/ && &has_command("$1")) {
+if ($config{'rndc_cmd'} =~ /^(\S+)/ && &has_command("$1") && $_[0] != 2) {
 	return 2;
 	}
-elsif ($config{'ndc_cmd'} =~ /^(\S+)/ && &has_command("$1")) {
+if ($config{'ndc_cmd'} =~ /^(\S+)/ && &has_command("$1") && $_[0] != 1) {
 	return 1;
 	}
-else {
-	return 0;
-	}
+return 0;
 }
 
 # get_pid_file()
@@ -1250,5 +1266,135 @@ if ($config{'zones_file'}) {
 return $config{'named_conf'};
 }
 
+# free_address_button(name, [form])
+sub free_address_button
+{
+local $form = defined($_[2]) ? $_[2] : 0;
+return "<input type=button onClick='ifield = document.forms[$form].$_[0]; chooser = window.open(\"free_chooser.cgi\", \"chooser\", \"toolbar=no,menubar=no,scrollbars=yes,width=150,height=500\"); chooser.ifield = ifield; window.ifield = ifield' value=\"...\">\n";
+}
+
+# create_slave_zone(name, master-ip, [view])
+# A convenience function for creating a new slave zone, if it doesn't exist
+# yet. Mainly useful for Virtualmin, to avoid excessive transfer of BIND
+# configuration data.
+# Returns 0 on success, 1 if BIND is not setup, 2 if the zone already exists,
+# or 3 if the view doesn't exist
+sub create_slave_zone
+{
+local $parent = &get_config_parent();
+local $conf = $parent->{'members'};
+local $opts = &find("options", $conf);
+if (!$opts) {
+	return 1;
+	}
+
+# Check if exists
+local @zones = &find("zone", $conf);
+local $v;
+foreach $v (&find("view", $conf)) {
+	push(@zones, &find("zone", $v->{'members'}));
+	}
+local ($z) = grep { $_->{'value'} eq $_[0] } @zones;
+return 2 if ($z);
+
+# Create it
+local $masters = { 'name' => 'masters',
+                   'type' => 1,
+                   'members' => [ { 'name' => $_[1] } ] };
+local $dir = { 'name' => 'zone',
+               'values' => [ $_[0] ],
+               'type' => 1,
+               'members' => [ { 'name' => 'type',
+                                'values' => [ 'slave' ] },
+                                $masters
+                            ]
+	     };
+local $base = $config{'slave_dir'} || &base_directory();
+local $file = $base."/".$_[0].".hosts";
+push(@{$dir->{'members'}}, { 'name' => 'file',
+			     'values' => [ $file ] } );
+if ($_[2]) {
+	# Create in a view
+	local $view = grep { $_->{'value'} eq $_[2] } &find("view", $conf);
+	return 3 if (!$view);
+	&save_directive($view, undef, [ $dir ], 0);
+	}
+else {
+	# Create at top level
+	&save_directive($parent, undef, [ $dir ], 0);
+	}
+&flush_file_lines();
+
+return 0;
+}
+
+# delete_zone(name, [view])
+# Delete one zone from named.conf
+# Returns 0 on success, or 1 if the zone was not found
+sub delete_zone
+{
+local $parent = &get_config_parent();
+local $conf = $parent->{'members'};
+local @zones;
+if ($_[1]) {
+	# Look in one view
+	local ($v) = grep { $_->{'value'} eq $_[1] } &find("view", $conf);
+	local @zones = &find("zone", $v->{'members'});
+	}
+else {
+	# Look in all views
+	@zones = &find("zone", $conf);
+	local $v;
+	foreach $v (&find("view", $conf)) {
+		push(@zones, &find("zone", $v->{'members'}));
+		}
+	}
+local ($z) = grep { $_->{'value'} eq $_[0] } @zones;
+return 1 if (!$z);
+
+&save_directive($parent, [ $z ], [ ]);
+&flush_file_lines();
+
+return 0;
+}
+
+# restart_bind()
+# A convenience function for re-starting BIND. Returns undef on success, or
+# an error message on failure.
+sub restart_bind
+{
+if ($config{'restart_cmd'}) {
+	local $out = &backquote_logged("$config{'restart_cmd'}2>&1 </dev/null");
+	if ($?) {
+		return &text('restart_ecmd', "<pre>$out</pre>");
+		}
+	}
+else {
+	local $pidfile = &get_pid_file();
+	local $pid;
+	open(PIDFILE, $pidfile);
+	chop($pid = <PIDFILE>);
+	close(PIDFILE);
+	if (!$pid || !&kill_logged('HUP', $pid)) {
+		return &text('restart_esig', $pid, $!);
+		}
+	}
+return undef;
+}
+
+# version_atleast(v1, v2, v3)
+sub version_atleast
+{
+local @vsp = split(/\./, $bind_version);
+local $i;
+for($i=0; $i<@vsp || $i<@_; $i++) {
+	return 0 if ($vsp[$i] < $_[$i]);
+	return 1 if ($vsp[$i] > $_[$i]);
+	}
+return 1;	# same!
+}
+
+
+
 1;
 
