@@ -41,7 +41,7 @@ if ($has_debian_iptables) {
 	return &init::action_status("iptables") == 2;
 	}
 else {
-	return &init::action_status("webmin-iptables");
+	return &init::action_status("webmin-iptables") == 2;
 	}
 }
 
