@@ -13,7 +13,7 @@ if (!-r $init_script) {
 	}
 if ($gconfig{'os_type'} ne 'trustix-linux') {
 	local $out = `$init_script status 2>&1`;
-	if ($out !~ /table:|INPUT|FORWARD|OUTPUT/) {
+	if ($out !~ /table:|INPUT|FORWARD|OUTPUT|is\s+stopped|firewall\s+stopped/) {
 		return &text('redhat_eoutput', "<tt>$init_script status</tt>");
 		}
 	}
