@@ -30,6 +30,7 @@ $POSTFIX_MODULE_VERSION = 5;
 
 do '../web-lib.pl';
 &init_config();
+require '../ui-lib.pl';
 %access = &get_module_acl();
 do 'aliases-lib.pl';
 
@@ -461,6 +462,7 @@ sub regenerate_any_table
     {
 	foreach $map (&get_maps_files(&get_real_value($_[0])))
 	{
+	    next unless $map;
 	    $out = `$config{'postfix_lookup_table_command'} -c $config_dir $map 2>&1`;
 	    if ($out) { &error(&text('regenerate_table_efailed', $map, $out)); }
 	}
