@@ -3,6 +3,7 @@
 
 do '../web-lib.pl';
 &init_config();
+require '../ui-lib.pl';
 
 # Load the site-specific information on the server executable
 &read_file("$module_config_directory/site", \%site);
@@ -453,28 +454,21 @@ foreach $m (sort { $a cmp $b } (@module_files)) {
 			}
 		}
         }
-print "<table width=100% cellpadding=5>\n";
+local (@titles, @links, @icons);
 for($i=0; $text{"type_$i"}; $i++) {
 	if ($etype{$i}) {
-		if ($c%5 == 0) { print "<tr>\n"; }
-		print "<td valign=top align=center width=20%>\n";
-		&generate_icon("images/type_icon_$i.gif", $text{"type_$i"},
-			       $_[1]."type=$i");
-		print "</td>\n";
-		if ($c%5 == 4) { print "<tr>\n"; }
-		$c++;
+		push(@links, $_[1]."type=$i");
+		push(@titles, $text{"type_$i"});
+		push(@icons, "images/type_icon_$i.gif");
 		}
 	}
 for($i=2; $i<@_; $i++) {
-	if ($c%5 == 0) { print "<tr>\n"; }
-	print "<td valign=top align=center width=20%>\n";
-	&generate_icon($_[$i]->{'icon'}, $_[$i]->{'name'}, $_[$i]->{'link'});
-	print "</td>\n";
-	if ($c%5 == 4) { print "<tr>\n"; }
-	$c++;
+	push(@links, $_[$i]->{'link'});
+	push(@titles, $_[$i]->{'name'});
+	push(@icons, $_[$i]->{'icon'});
 	}
-while($c++ % 5) { print "<td width=20%></td>\n"; }
-print "</table><p>\n";
+&icons_table(\@links, \@titles, \@icons, 5);
+print "<p>\n";
 }
 
 sub lock_proftpd_files
